import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import { is, fromJS } from 'immutable'
import { message } from 'antd'
import update from 'immutability-helper'

import Card from './card'
import Utils from '@/utils/utils.js'
import Action from './action'
import './index.scss'

const Container = ({list, parent, fields, handleList, handleMenu, deleteMenu, profileAction, handleStyle, updateMarks, dropButton, handleSubConfig }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)
    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })
    handleList(_cards)
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }
  
  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const editCard = id => {
    const { card } = findCard(id)
    handleMenu(card)
  }

  const copyCard = id => {
    const { card } = findCard(id)
    let copycard = fromJS(card).toJS()
    let _cards = fromJS(cards).toJS()

    if (card.eleType === 'button') {
      copycard.copyType = 'action'
    } else {
      copycard.copyType = 'customCardElement'
    }
    copycard.focus = true

    let _val = fromJS(copycard).toJS()

    if (_val.control) {
      delete _val.controlField
      delete _val.controlVal
    }

    copycard.uuid = Utils.getuuid()

    try {
      _val = window.btoa(window.encodeURIComponent(JSON.stringify(_val)))
    } catch (e) {
      console.warn('Stringify Failure')
      _val = ''
    }

    if (_val) {
      let oInput = document.createElement('input')
      oInput.value = _val
      document.body.appendChild(oInput)
      oInput.select()
      document.execCommand('Copy')
      document.body.removeChild(oInput)
      
      message.success('复制成功。')
    } else {
      message.warning('复制失败。')
    }

    if (card.eleType !== 'button' && !window.GLOB.CopyOnly) {
      _cards.push(copycard)
  
      handleList(_cards)
      handleMenu(copycard)
    }
  }

  const changeStyle = id => {
    const { card } = findCard(id)
    handleStyle(card)
  }

  const profileCard = id => {
    const { card } = findCard(id)
    profileAction(card)
  }

  const doubleClickCard = id => {
    const { card } = findCard(id)

    if (card.eleType !== 'button') {
      return
    }
    
    handleSubConfig(card)
  }

  const delCard = id => {
    const { card } = findCard(id)
    deleteMenu(card)
  }

  const [, drop] = useDrop({
    accept: 'action',
    drop(item) {
      if (item.$init) { // 拖拽添加
        let newcard = {}
        newcard.uuid = Utils.getuuid()
        newcard.focus = true

        // 显示列过滤
        if (parent.type === 'custom' && item.class !== 'element') {
          delete item.overIndex
          return
        } else if (parent.type === 'action' && item.class === 'element') {
          delete item.overIndex
          return
        }
        
        if (item.class === 'element') {
          newcard.eleType = item.value
          newcard.datatype = 'dynamic'
          newcard.height = 1

          if (item.value === 'splitline') {
            newcard.width = 24
            newcard.color = '#EBE9E9'
          } else if (item.value === 'tag') {
            newcard.style = {
              backgroundColor: 'var(--mk-sys-color1)',
              color: 'var(--mk-sys-color)',
              borderColor: 'var(--mk-sys-color)',
              borderWidth: '1px',
              borderRadius: '4px',
              paddingLeft: '7px',
              paddingRight: '7px',
              marginRight: '8px',
              marginBottom: '8px',
            }
          } else if (item.value === 'slider') {
            newcard.width = 24
            newcard.color = '#1890ff'
          }
        } else {
          newcard.eleType = 'button'
          newcard.label = 'button'
          newcard.verify = null
          newcard.show = 'link'
          newcard.Ot = 'requiredSgl'
          newcard.OpenType = item.value
          newcard.class = 'primary'

          if (newcard.OpenType === 'excelIn') {
            newcard.label = item.text
            newcard.class = 'border-dgreen'
            newcard.Ot = 'notRequired'
          } else if (item.subType === 'excelOut') {
            newcard.label = item.text
            newcard.execSuccess = 'never'
            newcard.class = 'dgreen'
          }
        }

        if (item.overIndex) {
          const { index } = findCard(item.overIndex)
          const _cards = update(cards, { $splice: [[index + 1, 0, newcard]] })
          handleList(_cards)
        } else {
          handleList([...cards, newcard])
        }

        handleMenu(newcard)

        delete item.overIndex
        return
      }

      const { index } = findCard(item.id)
      if (index > -1) return
      dropButton(item.id)
    }
  })

  return (
    <div ref={drop} className="ant-row card-detail-row">
      {cards.map(card => {
        if (card.eleType === 'button') {
          return (
            <Action
              id={card.uuid}
              key={card.uuid}
              card={card}
              copyCard={copyCard}
              moveCard={moveCard}
              editCard={editCard}
              changeStyle={changeStyle}
              profileCard={profileCard}
              doubleClickCard={doubleClickCard}
              delCard={delCard}
              findCard={findCard}
            />
          )
        } else {
          return (
            <Card
              id={card.uuid}
              key={card.uuid}
              card={card}
              parent={parent}
              fields={fields}
              moveCard={moveCard}
              copyCard={copyCard}
              editCard={editCard}
              updateMarks={updateMarks}
              changeStyle={changeStyle}
              delCard={delCard}
              findCard={findCard}
            />
          )
        }
      })}
    </div>
  )
}
export default Container
